
import lxml.html
from odoo import Command
from odoo.tests import tagged
from odoo.addons.base.tests.common import HttpCaseWithUserPortal


@tagged("post_install", "-at_install")
class TestPortalMyHome(HttpCaseWithUserPortal):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.quickstart_proj = cls.env["project.project"].create({
            "name": "Quickstart",
            "privacy_visibility": "portal",
            "allow_billable": True,
        })
        cls.product_success_pack_basic = (
            cls.env.ref("openerp_enterprise.product_pack_partner_starter_product_template")
            .product_variant_ids[:1]
        )
        cls.product_success_pack_basic.write({
            "detailed_type": "service",
            "service_tracking": "task_global_project",
            "project_id": cls.quickstart_proj.id,
        })

    def _create_success_pack_order(self, partner):
        sp_order = self.env["sale.order"].create({
            "partner_id": partner.id,
            "order_line": [
                Command.create({"product_id": self.product_success_pack_basic.id})
            ]
        })
        sp_order.action_confirm()
        return sp_order

    def test_old_successpack_redirect(self):
        self.authenticate("portal", "portal")

        sp_order = self._create_success_pack_order(self.partner_portal)
        self.assertFalse(
            sp_order.analytic_account_id,
            "No analytic account should be automatillcay created on SO "
            "confirmation for Success Packs"
        )

        # Manually force analytic account creation the old way for the purpose
        # of this test
        sp_order._create_analytic_account(prefix="TEST")
        analytic_account = sp_order.analytic_account_id

        old_pack_url = f"/my/pack/{analytic_account.id}"
        resp = self.url_open(old_pack_url, allow_redirects=False)
        self.assertEqual(resp.status_code, 303)
        self.assertTrue(
            sp_order.access_url in resp.headers["Location"],
            "Should redirect on the order"
        )

        sp_order2 = self._create_success_pack_order(self.partner_portal)
        sp_order2.analytic_account_id = analytic_account
        resp = self.url_open(old_pack_url, allow_redirects=False)
        self.assertEqual(resp.status_code, 303)
        self.assertTrue(
            "/my/tasks" in resp.headers["Location"],
            "Should redirect to tasks list",
        )

        resp = self.url_open(resp.headers["Location"])
        resp.raise_for_status()
        page_html = lxml.html.fromstring(resp.content)
        for task in (sp_order | sp_order2).tasks_ids:
            self.assertTrue(
                page_html.xpath("//a[contains(@href, '/my/tasks/%d')]" % task.id),
                "Task %s (id: %s) should be visible" % (task.display_name, task.id),
            )

    def test_portal_task_description_visiblity(self):
        sp_order = self._create_success_pack_order(self.partner_portal)
        task = sp_order.tasks_ids
        task.write({'description': '<p>Super Secret Description</p>'})

        self.assertTrue(task.project_id.show_task_description)
        self.authenticate('portal', 'portal')
        resp = self.url_open(task.access_url)
        resp.raise_for_status()
        page_html = lxml.html.fromstring(resp.content)
        self.assertTrue(
            page_html.xpath("//p[contains(text(), 'Super Secret Description')]"),
            "Task description should be visible",
        )

        task.project_id.show_task_description = False
        resp = self.url_open(task.access_url)
        resp.raise_for_status()
        page_html = lxml.html.fromstring(resp.content)
        self.assertFalse(
            page_html.xpath("//p[contains(text(), 'Super Secret Description')]"),
            "Task description should NOT be visible",
        )
